---
layout: section
color: indigo
---

# Core Concepts


---
layout: side-title
title: Single-Page Application
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Single-Page Application (SPA)

:: content ::

## What is an Single-Page Application (SPA)?

An ==SPA== is a web app that loads a single HTML page and dynamically updates the
content **without reloading the whole page**.

<v-click>
<div class="ns-c-tight mt-6">

### Benefits
-	Fast user experience (no full page reloads)
-	Smooth navigation like a desktop app
-	Ideal for modern web apps (e.g. Gmail, Trello)

</div>
</v-click>

<v-click>
<div class="ns-c-tight mt-6">

### Tradeoffs
- Slower first load (everything loads up front)
- Requires JavaScript to function properly
- SEO can be tricky without special setup

</div>
</v-click>


---
layout: side-title
title: Single-File Components
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Single-File Components (SFC)

:: content ::

## `*.vue` files

- A ==Vue SFC== is a special file format that encapsulates the component's logic
  (`JavaScript`), template (`HTML`), and styles (`CSS`) in a single file.


```vue {*}{lines:true}
<template>
  <p class="greeting">{{ greeting }}</p>
</template>

<script setup>
import { ref } from 'vue'
const greeting = ref('Hello World!')
</script>

<style>
.greeting {
  color: red;
  font-weight: bold;
}
</style>
```

---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Text Interpolation


- The most basic form of data binding is text interpolation using the
=="Mustache"== syntax.

```vue {monaco-run} {autorun: false}
<template>
  <span>Message: {{ msg }}</span>
</template>

<script setup>
  const msg = "Hello World"
</script>
```

<v-clicks>

- The mustache tag will be replaced with the value of the `msg` property from the
corresponding component instance.

- It will also be updated whenever the `msg` property changes.

</v-clicks>


---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Raw HTML

- ==Directives== are prefixed with `v-` to indicate that they are special
attributes provided by Vue

<v-clicks>

- In order to output real HTML, the `v-html` directive is used:

```vue {monaco-run} {autorun: false}
<template>
  <p><strong>Using text interpolation:</strong> {{ rawHtml }}</p>
  <p><strong>Using v-html directive:</strong> <span v-html="rawHtml"></span></p>
</template>
<script setup>
  const rawHtml = '<span style="color: red">This should be red.</span>'
</script>
```

<div class="flex justify-center items-center">

- Only use `v-html` on trusted content and never on user-provided content (*XSS vulnerabilities*).

<uim-exclamation-triangle class="text-xl text-red-400 animate-ping" />
</div>

</v-clicks>



---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Attribute Bindings

- Mustaches cannot be used inside HTML attributes.​ The `v-bind`
directive is used.


````md magic-move
```vue {*}{lines:true}
<template>
  <div v-bind:id="id"></div>
</template>
<script setup>
  const id="myDiv"
</script>
```

```vue {*}{lines:true}
<template>
  <div :id="id"></div>
</template>
<script setup>
  const id="myDiv"
</script>
```

```vue {*}{lines:true}
<template>
  <!-- vuejs 3.4+: same as v-bind:id="id" -->
  <div v-bind:id></div>
</template>
<script setup>
  const id="myDiv"
</script>
```

```vue {*}{lines:true}
<template>
  <!-- vuejs 3.4+: same as v-bind:id="id" -->
  <div :id></div>
</template>
<script setup>
  const id="myDiv"
</script>
```
````

- Vue will keep the element's `id` attribute in sync with the component's
`id` property

<v-click>

- If the bound value is `null` or `undefined`, then the attribute will be
removed from the rendered element.

</v-click>

---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Boolean Attributes

- Boolean attributes are attributes that can indicate `true` / `false` values by
their presence on an element.

```vue {monaco-run}
<template>
  <button :disabled="isDisabled" class="px-3 py-2 border border-black">
    Follow
  </button>
</template>

<script setup>
  const isDisabled=false
</script>
```

<v-clicks>

- The `disabled` attribute will be included if `isDisabled` has a
[**truthy**](https://developer.mozilla.org/en-US/docs/Glossary/Truthy) value or
is an empty string.

- For other [**falsy**](https://developer.mozilla.org/en-US/docs/Glossary/Falsy)
values the attribute will be omitted.

</v-clicks>

---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Binding Multiple Attributes

- Multiple attributes can be bind to a single element by using `v-bind`
**without an argument**.

```vue {*}{lines:true}
<template>
  <div v-bind="objectOfAttrs"></div>
</template>

<script setup>
  const objectOfAttrs = {
    id: 'container',
    class: 'wrapper',
    style: 'background-color:green'
  }
</script>
```

---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## JavaScript Expressions

- Vue supports the full power of JavaScript expressions inside all data bindings.

````md magic-move
```vue {*}{lines: true}
<template>
  {{ number + 1 }}
</template>
```

```vue {*}{lines: true}
<template>
  {{ number + 1 }}

  {{ ok ? 'YES' : 'NO' }}
</template>
```

```vue {*}{lines: true}
<template>
  {{ number + 1 }}

  {{ ok ? 'YES' : 'NO' }}

  {{ message.split(' ').reverse().join('') }}
</template>
```

```vue {*}{lines: true}
<template>
  {{ number + 1 }}

  {{ ok ? 'YES' : 'NO' }}

  {{ message.split('').reverse().join('') }}

  <div :id="`list-${id}`"></div>
</template>
```


```vue {*}{lines: true}
<template>
  {{ number + 1 }}

  {{ ok ? 'YES' : 'NO' }}

  {{ message.split('').reverse().join('') }}

  <div :id="`list-${id}`"></div>

  <time :title="toTitleDate(date)" :datetime="date">{{ formatDate(date) }}</time>
</template>
```
````

<v-click at="4">

- It is possible to call a component-exposed method inside a binding expression.

</v-click>



---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## JavaScript Expressions

<Warning />

- Each binding can only contain one single expression, i.e.,
**a piece of code that can be evaluated to a value**

```vue {*}{lines: true}
<template>
  <!-- this is a statement, not an expression: -->
  {{ var a = 1 }}

  <!-- flow control won't work either, use ternary expressions -->
  {{ if (ok) { return message } }}
</template>
```


---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Directives


- Directives are special attributes with the `v-` prefix
- Vue provides a number of [built-in directives](https://vuejs.org/api/built-in-directives.html)

<v-click>

- A directive's job is to reactively apply updates to the DOM when the value of its expression changes

```vue {monaco-run}
<template>
  <p v-if="seen">Now you see me</p>
</template>

<script setup>
  const seen=false
</script>
```

</v-click>


---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Directives

- Some directives can take an ==argument==, denoted by a **colon** after the directive name.

```vue {monaco-run}
<template>
  <a v-bind:href="url">Homepage</a>
</template>

<script setup>
  const url="https://lipn.univ-paris13.fr/"
</script>
```

---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Directives

- It is also possible to use a JavaScript expression in a directive.

```vue {monaco-run}
<template>
	<a v-bind:[attrName]="url">Homepage</a>
  <!-- <a :['' + attrName]="url"> ... </a> -->
</template>

<script setup>
  const attrName="href"
  const url="https://lipn.univ-paris13.fr/"
</script>
```

<v-click>
  <Warning />

  - Dynamic arguments are expected to evaluate to a string
  - The special value `null` can be used to explicitly remove the binding

</v-click>


---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Directives

- The `v-on` directive listens to DOM events, e.g., a click.

```vue {monaco-run}
<template>
  <button v-on:click="clickFn" class="btn-blue">
    Follow
  </button>
</template>

<script setup>
  function clickFn() {
    alert("clicked");
  }
</script>
```


---
layout: side-title
title: Template Syntax
color: indigo
align: cm
titlewidth: is-3
---
:: title ::
# Template Syntax

:: content ::

## Modifiers

- ==Modifiers== are special postfixes denoted by a **dot**, which indicate that a
directive should be bound in some special way

```vue
<!-- .prevent modifier tells the v-on directive to call event.preventDefault() -->
<form @submit.prevent="onSubmit">...</form>
```

<v-click>

  <ImageWithCredit src="./images/vue_directives.png" credit="https://vuejs.org/guide" class="mt-15 w-140"/>

</v-click>
