---
layout: section
color: indigo
---

# Reusable Components


---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reusable Components

:: content ::

## What is a component?

- Vue allows to split an app into ==independent== and ==reusable== pieces.
- A **component** encapsulates its own *structure*, *style*, and *behavior*

<ImageWithCredit src="./images/components.png" credit="https://medium.com/js-dojo/vue-data-flow-how-it-works-3ff316a7ffcd" class="mt-10"/>


---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reusable Components

:: content ::

## How to define a component?

- A Vue component in defined in a dedicated file using the `.vue` extension (SFC)

<v-click>

- It can be also defined as a plain JavaScript object containing Vue-specific
options

````md magic-move
```js
import { ref } from 'vue'

export default {
  setup() {
    const count = ref(0)
    return { count }
  },
  template: `
    <button @click="count++">
      You clicked me {{ count }} times.
    </button>`
}
```

```js
import { ref } from 'vue'

export default {
  setup() {
    const count = ref(0)
    return { count }
  },
  // Can also target an in-DOM template:
  template: '#my-template-element'
}
```
````

</v-click>

<v-click>

- Multiple components can be exported from the same file

</v-click>


---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reusable Components

:: content ::

## How to use a component?

- To use a child component, we need to import it in the parent component

```vue {*}{lines: true}
<template>
  <button @click="incr" class="btn-blue">Clicked {{ count }} times</button>
</template>
<script setup>
  import { ref } from 'vue'
  const count = ref(0)
  const incr = () => count.value++
</script>
```

```vue {*|2,5}{lines: true}
<template>
  <ButtonCounter />
</template>
<script setup>
  import ButtonCounter from './ButtonCounter.vue'
</script>
```

<div class="flex justify-center mt-6">
  <ButtonCounter />
</div>

---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reusable Components

:: content ::

## Declaring Props

- Components can **pass data** to their child components via **props**

```vue
<template>
  <Button label="Click me" />
</template>
```

<div class="flex justify-center my-6">
<Button label="Click me" />
</div>

<v-clicks>

- Enable **customization and reuse** of components with dynamic values
- Think of them like **function parameters**, but for components
- Props flow **one-way**: from **parent to child**

</v-clicks>



---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
clicks: 2
---

:: title ::
# Reusable Components

:: content ::

## Declaring Props

- `defineProps` declares the list of *props* accepted by a component

- It's only available inside `<script setup>` and doesn't need to
be explicitly imported

```vue {*|5}{lines: true}
<template>
  <button class="btn-blue">{{ label }}</button>
</template>
<script setup>
  defineProps(['label'])
</script>
```


<v-click>

- `defineProps` also returns an object that we can be accessed in JavaScript if needed.

```vue {*}{at:3}
<script setup>
const props = defineProps(['title'])
console.log(props.title)
</script>
```

</v-click>




---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
clicks: 2
---

:: title ::
# Reusable Components

:: content ::

## Declaring Props

- Props can also be declared using object syntax

- The `key` is the name of the prop, while the `value` should be the constructor function of the expected type.

```vue {*}{lines: true}
<script setup>
  defineProps({
    id: Number,
    title: String,
  })
</script>
```


<v-click>

- You can use `v-bind` without an argument to pass all the properties of an object as props

```vue {1|2}{lines: true}
<BlogPost v-bind="post" />
<BlogPost :id="post.id" :title="post.title" />
```

</v-click>



---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reusable Components

:: content ::

## Props Validation

- Components can specify requirements for their props.
- If a requirement is not met, Vue will warn you in the browser's JavaScript
console
- The type can be a **custom class** or one of the following native
constructors: `String`, `Number`, `Boolean`, `Array`, `Object`, `Date`,
`Function`, `Symbol`, `Error`


````md magic-move
```vue
<script setup>
  defineProps({
    // `null` and `undefined` values will allow any type
    propA: null,
    propB: undefined,
  })
<script>
```

```vue
<script setup>
  defineProps({
    // Basic type check
    propC: Number,
  })
<script>
```


```vue
<script setup>
  defineProps({
    // Multiple possible types
    propD: [String, Number],
  })
<script>
```

```vue
<script setup>
  defineProps({
    // Required string
    propE: {
      type: String,
      required: true
    },
  })
<script>
```

```vue
<script setup>
  defineProps({
    // Required but nullable string
    propF: {
      type: [String, null],
      required: true
    },
  })
<script>
```

```vue
<script setup>
  defineProps({
    // Number with a default value
    propG: {
      type: Number,
      default: 100
    },
  })
<script>
```

```vue
<script setup>
  defineProps({
    // Function with a default value
    propH: {
      type: Function,
      default() {
        return 'Default function'
      }
  })
<script>
```

```vue
<script setup>
  defineProps({
    // Object with a default value
    propI: {
      type: Object,
      // Object or array defaults must be returned from a function.
      default(rawProps) {
        return { message: 'hello' }
      }
    },
  })
<script>
```

```vue
<script setup>
  defineProps({
    // Custom validator function
    // full props passed as 2nd argument in 3.4+
    propJ: {
      validator(value, props) {
        return ['success', 'warning', 'danger'].includes(value)
      }
    },
  })
<script>
```
````


---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reusable Components

:: content ::

## Listening to Events

- The parent can choose to listen to any event on the child component instance
with `v-on`
- The child component can emit an event on itself by calling the built-in `$emit` method, passing the name of the event.

<div class="flex">
<ImageWithCredit src="./images/emits.png" credit="https://medium.com/js-dojo/vue-data-flow-how-it-works-3ff316a7ffcd" class="mt-5 w-65"/>

<div>
<v-click>
```vue
<template>
  <Button @btn-clicked="counter++" />
</template>
<script setup>
  import { ref } from 'vue'
  const counter = ref(0)
</script>
```
</v-click>
<v-click>
```vue
<template>
  <button @click="$emit('btn-clicked')">
    Click
  </button>
</template>
```
</v-click>
</div>
</div>


---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reusable Components

:: content ::

## Listening to Events

- The `defineEmits` macro can be used to declare emitted events
- The `defineEmits` macro **cannot** be used inside a function, it must be
placed directly within `<script setup>`
- It returns an emit function that is equivalent to the `$emit` method
- The first argument to `emit()` is the event name. Any additional arguments are
passed on to the event listener.


```vue
<template>
  <button class="btn-blue" @click="handleBtn">Click</button>
</template>
<script setup>
  const emit = defineEmits(['btn-clicked'])
  const handleBtn = () => emit('btn-clicked', 1, 'hello')
</script>
```


---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reusable Components

:: content ::

## Listening to Events

- An emitted event can be validated if it is defined with the object syntax
instead of the array syntax

- To add validation, the event is assigned a function that receives the
arguments passed to the emit call and returns a boolean.

````md magic-move
```vue
<script setup>
  const emit = defineEmits({
    click: null, // No validation
  })
</script>
```
```vue
<script setup>
  const emit = defineEmits(['submit'])

  function submitForm(email, password) {
    emit('submit', { email, password })
  }
</script>
```

```vue
<script setup>
  const emit = defineEmits({
    // Validate submit event
    submit: ({ email, password }) => email && password
  })

  function submitForm(email, password) {
    emit('submit', { email, password })
  }
</script>
```
````


---
layout: side-title
title: Reusable Components
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reusable Components

:: content ::

## Content Insertion in Components

- ==Slots== let you pass custom content into a component
- Think of it like a **placeholder** in the child component

```vue {*|3}{lines: true}
<template>
  <div class="p-4 rounded-lg border-l-4 border-red-500 bg-red-100 text-red-800">
    <slot />  <!-- slot outlet -->
  </div>
</template>
```

<v-click>

```vue {*}{lines: true}
<template>
  <AlertBox>
    Something went wrong. Please try again. <!-- slot content -->
  </AlertBox>
</template>
<script setup>
  import AlertBox from './AlertBox.vue'
</script>
```

<AlertBox>Something went wrong. Please try again.</AlertBox>

</v-click>