---
layout: side-title
title: List Rendering
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# List Rendering

:: content ::

## Directive `v-for`

- The `v-for` directive is used to render a list of items based on an array


```vue {monaco-run}
<template>
  <div class="flex justify-between items-start">
    <ul>
      <li v-for="item in items" class="my-0!">
        {{ item.name }}
      </li>
    </ul>
    <button @click='items.push({name: `Item ${items.length + 1}`})'
            class="btn-blue">Add</button>
  </div>
</template>
<script setup>
  import { ref } from 'vue'
  const items = ref([ { name: 'Item 1' } ])
</script>
```


---
layout: side-title
title: List Rendering
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# List Rendering

:: content ::

## Directive `v-for`

- It supports an optional second alias for the ==index== of the current item.

```vue {monaco-run}
<template>
  <div class="flex justify-between items-start">
    <ul>
      <li v-for="(item, index) in items" class="my-0!">
        {{ index }}: {{ item.name }}
      </li>
    </ul>
    <button @click='items.push({name: `Item ${items.length + 1}`})'
            class="btn-blue">Add</button>
  </div>
</template>
<script setup>
  import { ref } from 'vue'
  const items = ref([ { name: 'Item 1' } ])
</script>
```



---
layout: side-title
title: List Rendering
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# List Rendering

:: content ::

## Directive `v-for`

- You can use destructuring on the `item` alias.

```vue {monaco-run}
<template>
  <div class="flex justify-between items-start">
    <ul>
      <li v-for="({ name }, index) in items" class="my-0!">
        {{ index }}: {{ name }}
      </li>
    </ul>
    <button @click='items.push({name: `Item ${items.length + 1}`})'
            class="btn-blue">Add</button>
  </div>
</template>
<script setup>
  import { ref } from 'vue'
  const items = ref([ { name: 'Item 1' } ])
</script>
```



---
layout: side-title
title: List Rendering
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# List Rendering

:: content ::

## Directive `v-for`

- You can also use `v-for` to iterate through the properties of an object

```vue {monaco-run}
<template>
  <ul>
    <li v-for="(value, key, index) in book" class="my-0!">
      {{ index }}. <strong>{{ key }}:</strong> {{ value }}
    </li>
  </ul>
</template>
<script setup>
  import { reactive } from 'vue'
  const book = reactive({
    title: 'The Algorithm Design Manual',
    author: 'Steven S. Skiena',
    publishedAt: '2021-07-10'
  })
</script>
```


---
layout: side-title
title: List Rendering
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# List Rendering

:: content ::

## Directive `v-for`

- It can also take an integer to repeat the template *n* times using the range
`1...n`

```vue {monaco-run}
<template>
  <div class="grid grid-cols-5 gap-4">
    <div v-for="n in 5" :key="n" class="p-4 border border-gray-500 text-center">
      <div class="w-14 h-14 mx-auto bg-gray-300 rounded-full mb-2"></div>
      <p class="text-sm text-gray-600">Item {{ n }}</p>
    </div>
  </div>
</template>
```


---
layout: side-title
title: List Rendering
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# List Rendering

:: content ::

## Directive `v-for`

- Directives are attached to a single element. We can use the `<template>`
element as an invisible wrapper.

```vue {monaco-run}
<template>
  <div class="flex items-center space-x-4">
    <template v-for="user in users" :key="user.id">
      <img class="w-12 h-12 rounded-full" :src="user.avatar" :alt="user.name" />
      <span class="text-xs text-gray-700">{{ user.name }}</span>
    </template>
  </div>
</template>
<script setup>
const users = [
  { id: 1, name: 'Bob', avatar: 'https://placehold.co/100x100?text=Bob' },
  { id: 2, name: 'Eve', avatar: 'https://placehold.co/100x100?text=Eve' },
];
</script>
```
