---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Listening to Events

- The `v-on` directive is used to listen to DOM events and run some JavaScript
when they're triggered.

- **Inline handlers:** Inline JavaScript to be executed when the event is
  triggered.

```vue {monaco-run}
<template>
  <button @click="count++" class="btn-blue">Count is: {{ count }}</button>
</template>

<script setup>
  import { ref } from 'vue'
  const count = ref(0)
</script>
```


---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Listening to Events

- **Method handlers:** A property name or path that points to a method defined
  on the component.


```vue {monaco-run}
<template>
  <button @click="increment" class="btn-blue">Count is: {{ count }}</button>
</template>

<script setup>
  import { ref } from 'vue'
  const count = ref(0)

  function increment () {
    count.value++
  }
</script>
```


---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Listening to Events

- A method handler automatically receives the native DOM Event object that triggers it

```vue {monaco-run}
<template>
  <button @click="click" class="btn-blue">Click {{ tag }}</button>
</template>

<script setup>
  import { ref } from 'vue'
  const tag = ref("")

  function click(event){
    tag.value = event.target.tagName
  }
</script>
```



---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Listening to Events

- Instead of binding directly to a method name, we can also call methods in an
inline handler.

```vue {monaco-run}
<template>
  <button @click="say('hello')" class="btn-blue">Say Hello</button>
</template>

<script setup>
  import { ref } from 'vue'

  function say (message) {
    alert(message)
  }
</script>
```



---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Listening to Events

- If you need to access the original DOM event in an inline handler:
  1. you can pass it into a method using the special `$event` variable
  2. or use an inline arrow function

```vue {monaco-run}
<template>
  <!-- using $event special variable -->
  <button @click="say('hello', $event)" class="btn-blue">$event</button>

  <!-- using inline arrow function -->
  <button @click="(event) => say('hello', event)" class="btn-blue ml-2">function</button>
</template>
<script setup>
  import { ref } from 'vue'
  function say (message, event) {
    if (event) { event.preventDefault() }
    alert(message)
  }
</script>
```


---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Event Modifiers

- Event modifiers are special suffixes that modify the event behaviour without
writing extra JavaScript
- Event modifiers are directive postfixes denoted by a ==dot==.

<v-click>

````md magic-move
```vue
<!-- the click event's propagation will be stopped    -->
<!-- i.e., event.stopPropagation()                    -->
<a @click.stop="doSomething"></a>
```

```vue
<!-- the click event's propagation will be stopped    -->
<!-- i.e., event.stopPropagation()                    -->
<a @click.stop="doSomething"></a>

<!-- the submit event will no longer reload the page  -->
<!-- i.e, event.preventDefault()                      -->
<form @submit.prevent="submitForm">
  <button type="submit">Submit</button>
</form>
```

```vue
<!-- the click event's propagation will be stopped    -->
<!-- i.e., event.stopPropagation()                    -->
<a @click.stop="doSomething"></a>

<!-- the submit event will no longer reload the page  -->
<!-- i.e, event.preventDefault()                      -->
<form @submit.prevent="submitForm">
  <button type="submit">Submit</button>
</form>

<!-- Only triggers if clicked directly on the div (not a child) -->
<div @click.self="doThat">...</div>
```

```vue
<!-- the click event's propagation will be stopped    -->
<!-- i.e., event.stopPropagation()                    -->
<a @click.stop="doSomething"></a>

<!-- the submit event will no longer reload the page  -->
<!-- i.e, event.preventDefault()                      -->
<form @submit.prevent="submitForm">
  <button type="submit">Submit</button>
</form>

<!-- Only triggers if clicked directly on the div (not a child) -->
<div @click.self="doThat">...</div>

<!-- the click event will be triggered at most once   -->
<a @click.once="doThis"></a>
```
````

</v-click>



---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Event Modifiers: `.stop`

The click event's propagation will be stopped: `event.stopPropagation()`

```vue {monaco-run}
<template>
  <div @click="parentClick" class="block-indigo space-x-4">
    <span class="font-semibold">Parent Div</span>
    <button @click.stop="buttonClick" class="btn-blue">Child Button</button>
  </div>
</template>

<script setup>
function parentClick() { alert('Parent div clicked!'); }
function buttonClick() { alert('Button clicked!'); }
</script>
```


---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Event Modifiers: `.self`

Only triggers the handler if the event was fired on the element itself.

```vue {monaco-run} {autorun: false}
<template>
  <div  v-if="isOpen"
        @click.self="closeModal"
        class="fixed inset-0 bg-black bg-opacity-50 flex items-center justify-center z-50">
    <div class="bg-white rounded-lg p-6 w-96 shadow-lg">
      <p>Click outside this box to close the modal.</p>
      <button @click="closeModal" class="mt-4 btn-blue">Close</button>
    </div>
  </div>
</template>

<script setup>
import { ref } from 'vue';

const isOpen = ref(true);
const closeModal = () => isOpen.value = false
</script>
```



---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Event Modifiers: `.once`

The handler will be triggered only once. Vue automatically removes the listener
after the first trigger.

```vue {monaco-run}
<template>
  <div class="flex flex-col items-center justify-center space-y-6">
    <button @click.once="visible = !visible" class="btn-blue">
      Click Me
    </button>
    <div v-if="visible">🎉 You clicked the button!</div>
  </div>
</template>

<script setup>
import { ref } from 'vue';
const visible = ref(false);
</script>
```


---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Event Modifiers

- Modifiers can be chained

```vue
<a @click.stop.prevent="doThat"></a>
```

<v-click>

<Warning />

- <span v-mark.highlight.yellow="1">Order matters</span> when using modifiers
because the relevant code is generated in the same order

</v-click>

<v-click>

#### Example

- `@click.prevent.self` will prevent click's default action on the element
<span v-mark.underline.red="2">itself and its children</span>

</v-click>

<v-click>

- `@click.self.prevent` will <span v-mark.underline.red="3">only</span> prevent
click's default action on the <span v-mark.underline.red="3">element itself</span>.

</v-click>


---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Event Modifiers

- The `.capture`, `.once`, and `.passive` modifiers mirror the options of the native
`addEventListener` [method](https://developer.mozilla.org/en-US/docs/Web/API/EventTarget/addEventListener#options)

<v-click>

````md magic-move
```vue
<!-- use capture mode when adding the event listener     -->
<!-- i.e. an event targeting an inner element is handled -->
<!-- here before being handled by that element           -->
<div @click.capture="doThis">...</div>
```

```vue
<!-- use capture mode when adding the event listener     -->
<!-- i.e. an event targeting an inner element is handled -->
<!-- here before being handled by that element           -->
<div @click.capture="doThis">...</div>

<!-- the scroll event's default behavior (scrolling) will happen -->
<!-- immediately, instead of waiting for `onScroll` to complete  -->
<!-- in case it contains `event.preventDefault()`                -->
<div @scroll.passive="onScroll">...</div>
```
````

</v-click>

<v-click>

- The `.passive` modifier is typically used with touch event listeners for
[improving performance on mobile devices](https://developer.mozilla.org/en-US/docs/Web/API/EventTarget/addEventListener#improving_scroll_performance_using_passive_listeners).

</v-click>



---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Event Modifiers: `.capture`

Adds the event listener in capture mode instead of the default ==bubbling== (i.e.,
child -> parent).

```vue {monaco-run}
<template>
  <div @click.capture="handleParentClick" class="block-indigo space-y-2">
    <p class="font-bold text-lg text-blue-800">Parent Div (click capture)</p>
    <button @click="handleChildClick" class="btn-blue">Child Button</button>
  </div>
</template>

<script setup>
  const handleParentClick = () => alert('✅ Parent clicked (captured first)')
  const handleChildClick = () => alert('🟡 Child clicked')
</script>
```



---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Keys Modifiers

- Vue allows adding key modifiers for `@` when listening for key events.


```vue {monaco-run}
<template>
  <div class="bg-white p-3 w-full space-y-2">
    <input  v-model="text" @keyup.esc="clear" placeholder=""
            class="w-full border border-gray-400 p-2 focus:outline-none"/>
    <p> Press <strong>Enter</strong> to submit, <strong>Esc</strong> to clear</p>
  </div>
</template>

<script setup>
import { ref } from 'vue';

const text = ref('');
function clear() { text.value = ''; }
</script>
```



---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Keys Modifiers

- Any valid key names exposed via
[KeyboardEvent.key](https://developer.mozilla.org/en-US/docs/Web/API/UI_Events/Keyboard_event_key_values)
can be used as modifiers (kebab-case), e.g., `page-down`

<v-clicks>

- Vue provides aliases for the most commonly used keys: `.enter`, `.tab`,
`.delete`, `.esc`, `.space`, `.up`, `.down`, `.left`, `.right`

- When using the modifiers `.ctrl`, `.alt`, `.shift`, `.meta`, the event
listener is triggered only when the corresponding key is pressed.

</v-clicks>


<v-click>

````md magic-move
```vue
<!-- Alt + Enter -->
<input @keyup.alt.enter="clear" />
```

```vue
<!-- Alt + Enter -->
<input @keyup.alt.enter="clear" />

<!-- Ctrl + Click -->
<div @click.ctrl="doSomething">Do something</div>
```
````

</v-click>

---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Keys Modifiers

- The `.exact` modifier allows control of the exact combination of system
modifiers needed to trigger an event.

```vue {monaco-run}
<template>
<div class="flex gap-3 justify-center">
  <!-- this will fire even if Alt or Ctrl is also pressed -->
  <button @click.shift="onClick" class="btn-blue">Shift</button>

  <!-- this will only fire when Shift and no other keys are pressed -->
  <button @click.shift.exact="onClick" class="btn-blue">Shift Exact</button>

  <!-- this will only fire when no system modifiers are pressed -->
  <button @click.exact="onClick" class="btn-blue">No Modifiers</button>
</div>
</template>
<script setup>
const onClick = () => alert("Hello !")
</script>
```

---
layout: side-title
title: Event Handling
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Event Handling

:: content ::

## Mouse Button Modifiers

- Vue provides the ability to restrict event handlers to only trigger on specific mouse buttons: `.left`, `.right`, `.middle`


```vue {monaco-run}
<template>
  <div class="flex items-center space-x-4">
    <button class="btn-blue w-60"
      @click.left="message = 'Left click detected!'"
      @click.middle="message = ' Middle click detected!'"
      @click.right.prevent="message = 'Right click detected!'"
    > Click with mouse buttons </button>
    <div v-show="message"> {{ message }} </div>
  </div>
</template>

<script setup>
import { ref } from 'vue';
const message = ref('');
</script>
```


---
layout: side-title
title: Input Bindings
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Input Bindings

:: content ::

## Two-way binding

- How can we synchronize the value of `<input>` elements with corresponding
variables in JavaScript ?

```vue
<template>
  <input type="text" id="firstname" name="firstname">
</template>
```

<ImageWithCredit src="./images/monkey.jpg" credit="https://x.com/niggativebts/status/1388948646964703239" class="w-45 mt-8" />



---
layout: side-title
title: Input Bindings
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Input Bindings

:: content ::

## Two-way binding

- Using `v-bind` and `v-on` together, we can create ==two-way bindings== on form
input elements

```vue {monaco-run}
<template>
  <div class="flex items-center">
    <input :value="text" class="flex-grow input-full" placeholder="Write here ..."
           @input="event => text = event.target.value"/>
    <span class="ml-4 text-sm text-gray-500">{{ text.length }}</span>
  </div>
</template>
<script setup>
import { ref } from 'vue'
const text = ref('')
</script>
```


---
layout: side-title
title: Input Bindings
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Input Bindings

:: content ::

## `v-model` Directive

- Vue provides a directive, `v-model`, which is essentially syntactic sugar for the above.


```vue {monaco-run}
<template>
  <div class="flex items-center">
    <input v-model="text" class="flex-grow input-full" placeholder="Write here ..." />
    <span class="ml-4 text-sm text-gray-500">{{ text.length }}</span>
  </div>
</template>
<script setup>
import { ref } from 'vue'
const text = ref('')
</script>
```

<v-click>

- `v-model` works not only on text inputs, but also on other input types such as checkboxes, radio buttons, and select dropdowns.

</v-click>




---
layout: side-title
title: Input Bindings
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Input Bindings

:: content ::

## Value Binding

- For radio, checkbox and select options, the `v-model` binding values are usually static strings (or booleans for checkbox)


```vue {monaco-run}
<template>
  <div class="flex items-center justify-between bg-white p-3 shadow-md w-full">
    <label class="flex items-center space-x-2">
      <!-- `toggle` is either true or false -->
      <input type="checkbox" v-model="agreed" />
      <span>I agree to the terms</span>
    </label>
    <span class="text-sm text-gray-400">{{ agreed }}</span>
  </div>
</template>
<script setup>
import { ref } from 'vue'
const agreed = ref(false)
</script>
```



---
layout: side-title
title: Input Bindings
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Input Bindings

:: content ::

## Value Binding

- `v-bind` can be used to bind the value to a dynamic property

```vue {monaco-run}
<template>
  <div class="flex items-center justify-between bg-white p-3 shadow-md w-full">
    <label class="flex items-center space-x-2">
      <!-- `agreed` is either true or false -->
      <input type="checkbox" v-model="agreed" :true-value="valueT" :false-value="valueF"/>
      <span>I agree to the terms</span>
    </label>
    <span class="text-sm text-gray-400">{{ agreed }}</span>
  </div>
</template>
<script setup>
import { ref } from 'vue'
const agreed = ref('no')
const valueT = ref('yes')
const valueF = ref('no')
</script>
```



---
layout: side-title
title: Input Bindings
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Input Bindings

:: content ::

## Modifiers

- You can trim automatically whitespaces from user input (`.trim`) or typecast
automatically as a number (`.number`)

- By default, `v-model` updates the data on every **input event**.
`.lazy` modifies this behaviour to update only after the **change
event**


```vue {monaco-run}
<template>
  <div class="flex flex-col bg-white p-3 shadow-md space-y-2">
    <input v-model.lazy="text" type="text" class="input-full" />
    <span class="text-sm">Lazy value: <strong>{{ text }}</strong></span>
  </div>
</template>
<script setup>
import { ref } from 'vue'
const text = ref('')
</script>
```