---
layout: side-title
title: Reactivity Fundamentals (`ref`)
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reactivity Fundamentals `ref()`

:: content ::

## Declaring Reactive State with `ref()`

- The recommended way to declare reactive state is using the `ref()` function

```ts {monaco-run} {autorun:false }
import { ref } from 'vue'

const count = ref(0)
console.log(count)
```


---
layout: side-title
title: Reactivity Fundamentals `ref()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reactivity Fundamentals `ref()`

:: content ::

## Declaring Reactive State with `ref()`

- `ref()` takes the argument and returns it wrapped within a `ref` object with a
  `.value` property


```ts {monaco-run} {autorun:false }
import { ref } from 'vue'

const count = ref(0)

count.value++
console.log(`count -> ${count.value}`)
```



---
layout: side-title
title: Reactivity Fundamentals `ref()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reactivity Fundamentals `ref()`

:: content ::

## Declaring Reactive State with `ref()`


- Refs can hold **any value type**, including deeply nested objects, arrays, or
  JavaScript built-in data structures like `Map`.

- A `ref` will make its value ==deeply reactive==.

```ts {monaco-run} {autorun: false}
import { ref } from 'vue'

const obj = ref({
  nested: { count: 0 },
  arr: ['foo']
})

obj.value.nested.count++
console.log(obj.value)
```


---
layout: side-title
title: Reactivity Fundamentals `ref()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reactivity Fundamentals `ref()`

:: content ::

## Declaring Reactive State with `ref()`

```vue {monaco-run}
<script setup>
import { ref } from 'vue'
const counter = ref(1)

function inc() { counter.value++ }
function dec() { counter.value-- }
</script>

<template>
  <div class="text-lg text-white flex items-center justify-between w-full">
    <button class="bg-red-600 p2 hover:bg-red-500" @click="dec">-1</button>
    <span class="text-orange-400 text-4xl">{{ counter }}</span>
    <button class="bg-green-600 p2 hover:bg-green-500" @click="inc">+1</button>
  </div>
</template>
```

---
layout: side-title
title: Reactivity Fundamentals `reactive()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reactivity Fundamentals `reactive()`

:: content ::

## Declaring Reactive State with `reactive()`

- Unlike a `ref` which wraps the inner value in a special object, `reactive()`
  makes an ==object== itself reactive.

```ts {monaco-run} {autorun: false}
import { reactive } from 'vue'

const state = reactive({ count: 0 })

state.count++
console.log(state)
```

<v-clicks>

- `reactive()` converts the object deeply: nested objects are also wrapped with
  `reactive()` when accessed.

</v-clicks>


---
layout: side-title
title: Reactivity Fundamentals `reactive()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reactivity Fundamentals `reactive()`

:: content ::

## Declaring Reactive State with `reactive()`

- Reactive objects are ==JavaScript Proxies== and behave just like normal
  objects.

<v-click>

<div class="flex justify-center items-center my-6">
<uim-exclamation-triangle class="text-xl text-red-400 animate-ping " />
</div>


```ts {monaco-run} {autorun: false}
import { reactive } from 'vue'

const raw = {}
const proxy = reactive(raw)

// proxy is NOT equal to the original.
console.log(proxy === raw)

// calling reactive() on the same object returns the same proxy
// console.log(reactive(raw) === proxy)

// calling reactive() on a proxy returns itself
// console.log(reactive(proxy) === proxy)
```

</v-click>


---
layout: side-title
title: Reactivity Fundamentals `reactive()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reactivity Fundamentals `reactive()`

:: content ::

## Limitations of `reactive()`

- **Limited value types:** it only works for <span
v-mark.highlight.yellow="1">object types</span> (objects, arrays, etc). It
cannot hold primitive types such as string, number or boolean.

<v-click>

- **Cannot replace entire object:** we can't easily "replace" a reactive object
  because the reactivity connection to the first reference is lost.

```ts {*}{lines:true}
import { reactive } from 'vue'
let state = reactive({ count: 0 })

// the above reference ({ count: 0 }) is no longer being tracked
// (reactivity connection is lost!)
state = reactive({ count: 1 })
```

</v-click>


---
layout: side-title
title: Reactivity Fundamentals `reactive()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reactivity Fundamentals `reactive()`

:: content ::

## Limitations of `reactive()`


- **Not destructure-friendly:** when we destructure a reactive object's
  primitive type property, we will lose the reactivity connection

```ts {monaco-run} {autorun: false}
import { reactive } from 'vue'

const state = reactive({ count: 0 })

// count is disconnected from state.count when destructured.
let { count } = state

// it does not affect original state
count++

console.log(state)
```



---
layout: side-title
title: Reactivity Fundamentals `reactive()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Reactivity Fundamentals `reactive()`

:: content ::

## Limitations of `reactive()`


- **Not destructure-friendly:** when we destructure a reactive object's
  primitive type property, we will lose the reactivity connection

```js {monaco-run} {autorun: false}
import { reactive } from 'vue'

const state = reactive({ count: 0 })

// the function receives a plain number and won't be able to track changes
function increment(value){ value++ }
increment(state.count)

// we have to pass the entire object in to retain reactivity
// function incrementObj(obj){ obj.count++ }
// incrementObj(state)

console.log(state)
```


---
layout: side-title
title: Computed Properties `computed()`
color: indigo
align: cm
titlewidth: is-3
---


:: title ::
# Computed Properties `computed()`

:: content ::

## Don't repeat yourself (==DRY== Principle)

- We don't want to repeat ourselves if we need to include a calculation in
the template more than once.

```vue {monaco-run}
<script setup>
import { ref } from 'vue'
const newItem = ref("")
</script>

<template>
<div class="flex items-center">
  <input v-model="newItem" placeholder="Enter your text..."
          class="flex-grow p-3 border border-gray-300 rounded-md"/>
  <span class="ml-4 text-sm text-gray-500">{{ newItem.length }}</span>
</div>
</template>
```


---
layout: side-title
title: Computed Properties `computed()`
color: indigo
align: cm
titlewidth: is-3
---


:: title ::
# Computed Properties `computed()`

:: content ::

## Don't repeat yourself (==DRY== Principle)

- For complex logic that includes ==reactive data==, it is recommended to use a
**computed property**

<v-click>

- A <span v-mark.box.yellow="1">computed property</span> automatically tracks its reactive dependencies.

```vue {monaco-run}
<script setup>
import { ref, computed } from 'vue'
const newItem = ref("")
const counter = computed(() => newItem.value.length)
</script>

<template>
<div class="flex items-center">
  <input v-model="newItem" placeholder="Enter your text..."
          class="flex-grow p-3 border border-gray-300 rounded-md"/>
  <span class="ml-4 text-sm text-gray-500">{{ counter }}</span>
</div>
</template>
```

</v-click>

---
layout: side-title
title: Computed Properties `computed()`
color: indigo
align: cm
titlewidth: is-3
---


:: title ::
# Computed Properties `computed()`

:: content ::

## Computed Caching vs. Methods

- We can define the same function as a method.

```vue {monaco-run}
<script setup>
import { ref } from 'vue'
const newItem = ref("")
const counter = () => newItem.value.length
</script>

<template>
<div class="flex items-center">
  <input v-model="newItem" placeholder="Enter your text..."
          class="flex-grow p-3 border border-gray-300 rounded-md"/>
  <span class="ml-4 text-sm text-gray-500">{{ counter() }}</span>
</div>
</template>
```

<v-clicks>

- **However,** A computed property will only ==re-evaluate== when some of its
reactive dependencies have changed

</v-clicks>


---
layout: side-title
title: Computed Properties `computed()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Computed Properties `computed()`

:: content ::

## Computed Caching vs. Methods

```vue {monaco-run}
<script setup>
import { ref, computed, getCurrentInstance } from 'vue'

// Used to force a re-render
const { proxy } = getCurrentInstance()
const forceRender = () => proxy.$forceUpdate()

const now = computed(() => new Date().toISOString()) // Computed property
const getNow = () => new Date().toISOString()        // Method
</script>

<template>
  <div class="flex justify-center items-center space-x-6">
    <span><strong>Computed time:</strong> {{ now }}</span>
    <span><strong>Method time:</strong> {{ getNow() }}</span>
    <button @click="forceRender" class="btn-blue">
      Refresh
    </button>
  </div>
</template>
```


---
layout: side-title
title: Computed Properties `computed()`
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Computed Properties `computed()`

:: content ::

## Best Practices

- **Getters should be side-effect free:** its only responsibility should be
<span v-mark.box.yellow="1">computing and returning</span> that value

<v-click>

- **Avoid mutating computed value:** a computed return value should be treated
as <span v-mark.box.yellow="2">read-only and never be mutated</span>

</v-click>

<img src="/images/best_practices_yoda.jpg" class="w-50 mx-auto">


---
layout: side-title
title: Watchers
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Watchers

:: content ::

## What is a watcher?

- A watcher triggers a callback **whenever** a reactive state changes
- Useful when you want to perform ==side effects== (e.g., react to props or route changes)

```vue {monaco-run}
<template>
  <button @click="count++" class="btn-blue">Count + 1</button>
</template>

<script setup>
import { ref, watch } from 'vue'
const count = ref(0)

watch(count, (newVal, oldVal) => {
  alert(`Count changed from ${oldVal} to ${newVal}`)
})
</script>
```


---
layout: side-title
title: Watchers
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Watchers

:: content ::

## Watch Source Types


<div class="ns-c-tight mb-4">

- The first argument of the `watch` function can be different types of reactive
"sources":
  - a ref (including computed refs),
  - a reactive object,
  - a getter function, or
  - an array of multiple sources

</div>

````md magic-move
```vue {*}{lines:true}
<script setup>
import { ref, watch } from 'vue'
const x = ref(0)

// a ref
watch(x, (newX) => {
  console.log(`x is ${newX}`)
})
</script>
```

```vue {*}{lines:true}
<script setup>
import { ref, watch } from 'vue'
const x = ref(0)
const y = ref(0)

// getter
watch(
  () => x.value + y.value,
  (sum) => {
    console.log(`sum of x + y is: ${sum}`)
  }
)
</script>
```

```vue {*}{lines:true}
<script setup>
import { ref, watch } from 'vue'
const x = ref(0)
const y = ref(0)

// array of multiple sources
watch([x, () => y.value], ([newX, newY]) => {
  console.log(`x is ${newX} and y is ${newY}`)
})
</script>
```
````



---
layout: side-title
title: Watchers
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Watchers

:: content ::

## Watch Source Types

- `watch()` does **not** detect changes in properties of reactive objects.

````md magic-move
```vue {*}{lines:true}
<script setup>
import { reactive, watch } from 'vue'
const obj = reactive({ count: 0 })

// this won't work because we are passing a number to watch()
watch(obj.count, (count) => {
  console.log(`Count is: ${count}`)
})
</script>
```

```vue {*}{lines:true}
<script setup>
import { reactive, watch } from 'vue'
const obj = reactive({ count: 0 })

// instead, use a getter
watch(
  () => obj.count,
  (count) => {
    console.log(`Count is: ${count}`)
  }
)
</script>
```
````




---
layout: side-title
title: Watchers
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Watchers

:: content ::

## Watch Source Types

- When directly watching a reactive object, the watcher is automatically in
`deep mode`

````md magic-move
```vue {*}{lines:true}
<script setup>
import { reactive, watch } from 'vue'
const state = reactive({ count: 0 })

watch(state, () => {
  /* triggers on deep mutation to state */
})
</script>
```
```vue {*}{lines:true}
<script setup>
import { reactive, watch } from 'vue'
const state = reactive({ count: 0 })

watch(
  () => state,
  (newValue, oldValue) => {
    // newValue === oldValue
  },
  { deep: true }
)
</script>
```
````


---
layout: side-title
title: Watchers
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Watchers

:: content ::

## Eager Watchers

- By default, the callback of a `watch` won't be called until the watched
source has changed

<v-click>

- We can force a watcher's callback to be executed immediately by passing the
`immediate: true` option


```vue {monaco-run} {autorun:false}
<template>
  <button @click="count++" class="btn-blue">Count + 1</button>
</template>

<script setup>
import { ref, watch } from 'vue'
const count = ref(0)

watch(count,
  (newVal, oldVal) => { alert(`Count changed from ${oldVal} to ${newVal}`) },
  { immediate: true })
</script>
```

</v-click>


---
layout: side-title
title: Watchers
color: indigo
align: cm
titlewidth: is-3
---

:: title ::
# Watchers

:: content ::

## Once Watchers

- If you want the callback to trigger **only once** when the source changes, use
the `once: true` option.


```vue {monaco-run}
<template>
  <button @click="count++" class="btn-blue">Count + 1</button>
</template>

<script setup>
import { ref, watch } from 'vue'
const count = ref(0)

watch(count,
  (newVal, oldVal) => { alert(`Count changed from ${oldVal} to ${newVal}`) },
  { once: true })
</script>
```